@@ -170,21 +170,25 @@ new_fluid_jack_client(fluid_settings_t *settings, int isaudio, void *driver)
             ((!isaudio && last_client->midi_driver == NULL) || (isaudio && last_client->audio_driver == NULL)))
     {
         client_ref = last_client;
-        last_client = NULL;         /* No more pairing for this client */
 
         /* Register ports */
-        if(fluid_jack_client_register_ports(driver, isaudio, client_ref->client, settings) != FLUID_OK)
+        if(fluid_jack_client_register_ports(driver, isaudio, client_ref->client, settings) == FLUID_OK)
         {
-            goto error_recovery;
-        }
+            last_client = NULL; /* No more pairing for this client */
 
-        if(isaudio)
-        {
-            fluid_atomic_pointer_set(&client_ref->audio_driver, driver);
+            if(isaudio)
+            {
+                fluid_atomic_pointer_set(&client_ref->audio_driver, driver);
+            }
+            else
+            {
+                fluid_atomic_pointer_set(&client_ref->midi_driver, driver);
+            }
         }
         else
         {
-            fluid_atomic_pointer_set(&client_ref->midi_driver, driver);
+            // do not free client_ref and do not goto error_recovery
+            // client_ref is being used by another audio or midi driver. Freeing it here will create a double free.
         }
 
         fluid_mutex_unlock(last_client_mutex);        /* -- unlock last_client */
@@ -484,7 +488,6 @@ fluid_jack_client_register_ports(void *driver, int isaudio, jack_client_t *clien
         }
     }
 
-
     /* Adjust sample rate to match JACK's */
     jack_srate = jack_get_sample_rate(client);
     FLUID_LOG(FLUID_DBG, "Jack engine sample rate: %lu", jack_srate);
@@ -493,15 +496,21 @@ fluid_jack_client_register_ports(void *driver, int isaudio, jack_client_t *clien
 
     if((unsigned long)sample_rate != jack_srate)
     {
-        FLUID_LOG(FLUID_INFO, "Jack sample rate mismatch, adjusting."
+        fluid_synth_t* synth;
+        if(fluid_jack_obtain_synth(settings, &synth) == FLUID_OK)
+        {
+            FLUID_LOG(FLUID_INFO, "Jack sample rate mismatch, adjusting."
                   " (synth.sample-rate=%lu, jackd=%lu)", (unsigned long)sample_rate, jack_srate);
-        fluid_settings_setnum(settings, "synth.sample-rate", jack_srate);
-    }
-
-    /* Changing sample rate is non RT, so make sure we process it and/or other things now */
-    if(dev->callback == NULL)
-    {
-        fluid_synth_process_event_queue(dev->data);
+            fluid_synth_set_sample_rate(synth, jack_srate);
+            /* Changing sample rate is non RT, so make sure we process it and/or other things now */
+            fluid_synth_process_event_queue(synth);
+        }
+        else
+        {
+            FLUID_LOG(FLUID_WARN, "Jack sample rate mismatch (synth.sample-rate=%lu, jackd=%lu)"
+            " impossible to adjust, because the settings object provided to new_fluid_audio_driver2() was not used to create a synth."
+            , (unsigned long)sample_rate, jack_srate);
+        }
     }
 
     return FLUID_OK;
@@ -865,7 +874,7 @@ new_fluid_jack_midi_driver(fluid_settings_t *settings,
     }
 
     return (fluid_midi_driver_t *)dev;
-    
+
 error_recovery:
     delete_fluid_jack_midi_driver((fluid_midi_driver_t *)dev);
     return NULL;
@@ -887,4 +896,18 @@ delete_fluid_jack_midi_driver(fluid_midi_driver_t *p)
     FLUID_FREE(dev);
 }
 
+int fluid_jack_obtain_synth(fluid_settings_t *settings, fluid_synth_t **synth)
+{
+    void *data;
+
+    if(!fluid_settings_is_realtime(settings, "synth.gain") ||
+       (data = fluid_settings_get_user_data(settings, "synth.gain")) == NULL)
+    {
+        return FLUID_FAILED;
+    }
+
+    *synth = data;
+    return FLUID_OK;
+}
+
 #endif /* JACK_SUPPORT */
