@@ -379,7 +379,7 @@ fluid_rvoice_buffers_mix(fluid_rvoice_buffers_t *buffers,
     int bufcount = buffers->count;
     int i, dsp_i;
 
-    /* if there is nothing to mix, return immediatly */
+    /* if there is nothing to mix, return immediately */
     if(sample_count <= 0 || dest_bufcount <= 0)
     {
         return;
@@ -672,7 +672,7 @@ fluid_mixer_buffers_init(fluid_mixer_buffers_t *buffers, fluid_rvoice_mixer_t *m
 DECLARE_FLUID_RVOICE_FUNCTION(fluid_rvoice_mixer_set_samplerate)
 {
     fluid_rvoice_mixer_t *mixer = obj;
-    fluid_real_t samplerate = param[1].real; // becausee fluid_synth_update_mixer() puts real into arg2
+    fluid_real_t samplerate = param[1].real; // because fluid_synth_update_mixer() puts real into arg2
 
     int i;
     for(i = 0; i < mixer->fx_units; i++)
